@@ -33,6 +33,11 @@
 #' @param b0 scalar. Prior value for the beta rate. Default is 1.
 #' @param two_side logical. Indicator of two-sided test for the discount
 #'   function. Default value is TRUE.
+#' @param method character. Analysis method with respect to estimation of the weight
+#'   paramter alpha. Default value "\code{fixed}" estimates alpha once and holds it fixed
+#'   throughout the analysis. Alternative method "\code{mc}" estimates alpha for each
+#'   Monte Carlo iteration. See the the \code{bdpbinomial} vignette \cr
+#'   \code{vignette("bdpbinomial-vignette", package="bayesDP")} for more details.
 #' @details \code{bdpbinomial} uses a two-stage approach for determining the
 #'   strength of historical data in estimation of a binomial count mean outcome.
 #'   In the first stage, a Weibull distribution function is used as a
@@ -153,7 +158,7 @@
 #'
 #' @rdname bdpbinomial
 #' @import methods
-#' @importFrom stats sd density is.empty.model median model.offset model.response pweibull quantile rbeta rgamma rnorm var vcov
+#' @importFrom stats sd density is.empty.model median model.offset model.response pweibull quantile rbeta rgamma rnorm var vcov pchisq
 #' @aliases bdpbinomial,ANY-method
 #' @export bdpbinomial
 bdpbinomial <- setClass("bdpbinomial",
@@ -177,7 +182,8 @@ setGeneric("bdpbinomial",
                     number_mcmc   = 10000,
                     weibull_scale = 0.135,
                     weibull_shape = 3,
-                    two_side      = TRUE){
+                    two_side      = TRUE,
+                    method        = "fixed"){
              standardGeneric("bdpbinomial")
            })
 
@@ -198,7 +204,8 @@ setMethod("bdpbinomial",
                    number_mcmc   = 10000,
                    weibull_scale = 0.135,
                    weibull_shape = 3,
-                   two_side      = TRUE){
+                   two_side      = TRUE,
+                   method        = "fixed"){
 
 
   ################################################################################
@@ -306,7 +313,8 @@ setMethod("bdpbinomial",
     number_mcmc   = number_mcmc,
     weibull_scale = weibull_scale[1],
     weibull_shape = weibull_shape[1],
-    two_side      = two_side)
+    two_side      = two_side,
+    method        = method)
 
   if(arm2){
     posterior_control <- posterior_binomial(
@@ -321,7 +329,8 @@ setMethod("bdpbinomial",
       number_mcmc   = number_mcmc,
       weibull_scale = weibull_scale[2],
       weibull_shape = weibull_shape[2],
-      two_side      = two_side)
+      two_side      = two_side,
+      method        = method)
   } else{
     posterior_control <- NULL
   }
@@ -343,6 +352,7 @@ setMethod("bdpbinomial",
                 weibull_scale = weibull_scale,
                 weibull_shape = weibull_shape,
                 two_side      = two_side,
+                method        = method,
                 arm2          = arm2,
                 intent        = paste(intent,collapse=", "))
 
@@ -366,7 +376,7 @@ setMethod("bdpbinomial",
 ################################################################################
 posterior_binomial <- function(y, N, y0, N0, alpha_max, fix_alpha, a0, b0,
                                number_mcmc, weibull_scale, weibull_shape,
-                               two_side){
+                               two_side, method){
 
   ### Compute posterior(s) of current (flat) and historical (prior) data
   ### with non-informative priors
@@ -390,13 +400,21 @@ posterior_binomial <- function(y, N, y0, N0, alpha_max, fix_alpha, a0, b0,
   if(!is.null(N) & !is.null(N0)){
 
     ### Test of model vs real
-    p_hat <- mean(posterior_flat < prior)   # larger is higher failure
+    if(method == "fixed"){
+      p_hat <- mean(posterior_flat < prior)   # larger is higher failure
+    } else if(method == "mc"){
+      v     <- 1/((y + a0 - 1)/posterior_flat^2 + (N-y+b0-1)/(posterior_flat-1)^2)
+      v0    <- 1/((y0 + a0 - 1)/prior^2 + (N0-y0+b0-1)/(prior-1)^2)
+      Z     <- (posterior_flat-prior)^2 / (v+v0)
+      p_hat <- pchisq(Z,1,lower.tail=FALSE)
+    }
+
 
     ### Number of effective sample size given shape and scale discount function
     if(fix_alpha == TRUE){
       alpha_discount <- alpha_max
     } else{
-      if (!two_side) {
+      if (!two_side | method == "mc") {
         alpha_discount <- pweibull(p_hat, shape=weibull_shape,
                                    scale=weibull_scale)*alpha_max
       } else if (two_side){
